//----------------------------------------------------------------------------
//  Project Test_At3dx_Lights
//  MicroMouse Productions
//  Copyright  2011-2012. All Rights Reserved.
//  Author: Wayne Hogue
//
//----------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <stdlib.h>
#include <stdio.h>

#include "Test_At3dx_LightsApp.h"
#include "at3dx_my_functions.h"
#include "at3ddll.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTest_At3dx_LightsForm *Test_At3dx_LightsForm;
//---------------------------------------------------------------------------

__fastcall TTest_At3dx_LightsForm::TTest_At3dx_LightsForm(TComponent* Owner)
  : TForm(Owner)
{
  dllOpen    = 0;
  fileLoaded = 0;

  memset(inName, 0, sizeof(char) * MAX_PATH);

  inputFileType = 0;

  // Display textures
  //
  useTexturesFlag = 1;

  // Let texture colors be affected by the lighting
  //
  oglReplaceFlag = 0;

  // When displaying textures use white as the diffuse color
  // instead of the assigned diffuse color for the layer
  //
  useWhiteBaseFlag = 1;

  // Reverse direction of pan
  //
  reversePanFlag = 1;

  // Reverse direction of zoom
  //
  reverseZoomFlag = 1;

  // Default setting for camera cursor
  //
  alwaysShowCameraCursorFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::FormCreate(TObject *Sender)
{
  // calls to DLL functions are blocked by dllOpen flag being zero
  //

  // by default, only Light 0 is turned on
  //
  Light0->Checked = 1;
  Light1->Checked = 0;

  // set intial file type to read
  //
  if (!inputFileType)
  {
    TypeKmz->Checked = 1;
  }
  else
  {
    TypeObj->Checked = 1;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::FormDestroy(TObject *Sender)
{
  // free all memory used by the DLL
	//
  if (dllOpen) Stop_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::DoStartDllClick(TObject *Sender)
{
  Start_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::DoStopDllClick(TObject *Sender)
{
  Stop_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::Read3dFileClick(TObject *Sender)
{
  Read_3D_File();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::Start_DLL(void)
{
  if (dllOpen) return;

  dllOpen = At3d_Start();

  if (!dllOpen) return;

  // Defaults for this DLL and other software can change without notice
  // Recommended practice is to set the defaults you want for your program
  // for those DLL features you will be using
  //
  // Following are the defaults chosen for this Demo
  //
  // To detect mouse scroll wheel messages the DLL has to know the
  // handle to the main window of the application which
  // receives all the messages from the Windows Operating System
  //
  At3d_SetAppHandle(Handle);

  // Set size of DLL window
  //
  At3d_Size(600, 500);

  // Set position of DLL window
  //
  At3d_Position(2, 2);

  // Make DLL window a child of a window in the application
  //
  At3d_SetParent(ViewPanel->Handle);

  // If the DLL window does not have a parent
  // At3d_SetParent(0);
  // a button will appear on the Windows Taskbar
  // use this function to set the name that will appear
  //
  At3d_SetCaption("DLL (at3d)");

  // use the OpenGL viewer
  //
  At3d_WriteFlag(LIBFLAG_USE_OPENGL, 1);

  // light both sides of polygons
  //
  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, 1);

  // do NOT cull back faces
  //
  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES, 0);

  UseTextures->Checked = (useTexturesFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, useTexturesFlag);

  // do NOT display the default texture when
  // - the texture file can not be found
  // - the texture file format is not supported
  // - no texture file was assigned
  //
  At3d_WriteFlag(LIBFLAG_OGL_USE_DEFAULT_TEXTURE, 0);

  UseOglReplace->Checked = (oglReplaceFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_REPLACE, oglReplaceFlag);

  UseWhiteBase->Checked = (useWhiteBaseFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_WHITE_BASE, useWhiteBaseFlag);

  ReversePan->Checked = (reversePanFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_PAN_DIRECTION, reversePanFlag);

  ReverseZoom->Checked = (reverseZoomFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_ZOOM_DIRECTION, reverseZoomFlag);

  AlwaysShowCameraCursor->Checked = (alwaysShowCameraCursorFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, alwaysShowCameraCursorFlag);

  SetLight0();
  SetLight1();

  // Make DLL window visible
  //
  At3d_Show();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::Stop_DLL(void)
{
  if (!dllOpen) return;

  // free all memory used by the DLL
	//
  At3d_Stop();
  dllOpen = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::Read_3D_File(void)
{
  if (!dllOpen) return;

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
	char dir0[_MAX_DIR];
	char name0[_MAX_FNAME];
	char ext0[_MAX_EXT];
   
  if (inName[0] != 0)
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  char* inputNamePtr;
  char* inputExtensionPtr;
  __int32 inputType;

  if (!inputFileType)
  {
    inputNamePtr      = "Google Earth";
    inputExtensionPtr = ".kmz";
    inputType         = INTYPE_GOOGLE_EARTH;
  }
  else
  {    
    inputNamePtr      = "Wavefront";
    inputExtensionPtr = ".obj";
    inputType         = INTYPE_WAVEFRONT;
  }

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(0);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir = path;
  dlg->FileName = name0;
  dlg->Filter = filter;
  dlg->FilterIndex = 0;

 	if (dlg->Execute() == IDOK)
  {
    my_strcpy(inName, MAX_PATH, dlg->FileName.c_str());

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // read file
    //
    fileLoaded = At3d_Read(inputType, inName, 1.0);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::Light0Click(TObject *Sender)
{
  if (dllOpen) SetLight0();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::SetLight0(void)
{
  if (!dllOpen) return;

  __int32 enabledFlag = (Light0->Checked) ? 1 : 0;

  float ambient[4];
  float diffuse[4];
  float specular[4];
  float position[3];

  ambient[0] = 0.0;
  ambient[1] = 0.0;
  ambient[2] = 0.0;
  ambient[3] = 1.0;

  diffuse[0] = 1.0;
  diffuse[1] = 1.0;
  diffuse[2] = 1.0;
  diffuse[3] = 1.0;

  specular[0] = 0.0;
  specular[1] = 0.0;
  specular[2] = 0.0;
  specular[3] = 1.0;

  position[0] = 0.0;
  position[1] = 0.0;
  position[2] = 1.0;

  __int32 result = At3d_Set_OGL_Directional_Light(0, enabledFlag,
                                                  ambient, diffuse,
                                                  specular, position);

  // to update OpenGL light settings immediately do a redraw
  //
  if (result) At3d_DrawWireframe();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::Light1Click(TObject *Sender)
{
  if (dllOpen) SetLight1();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::SetLight1(void)
{
  if (!dllOpen) return;

  __int32 enabledFlag = (Light1->Checked) ? 1 : 0;

  float ambient[4];
  float diffuse[4];
  float specular[4];
  float position[3];

  ambient[0] = 0.0;
  ambient[1] = 0.0;
  ambient[2] = 0.0;
  ambient[3] = 1.0;

  diffuse[0] = 0.5;
  diffuse[1] = 0.5;
  diffuse[2] = 0.5;
  diffuse[3] = 1.0;

  specular[0] = 0.0;
  specular[1] = 0.0;
  specular[2] = 0.0;
  specular[3] = 1.0;

  position[0] = 0.0;
  position[1] = 1.0;
  position[2] = 0.5;

  __int32 result = At3d_Set_OGL_Directional_Light(1, enabledFlag,
                                                  ambient, diffuse,
                                                  specular, position);

  // to update OpenGL light settings immediately do a redraw
  //
  if (result) At3d_DrawWireframe();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::TypeKmzClick(TObject *Sender)
{
  inputFileType = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::TypeObjClick(TObject *Sender)
{
  inputFileType = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::UseOglReplaceClick(TObject *Sender)
{
  if (!dllOpen) return;

  oglReplaceFlag = (UseOglReplace->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_REPLACE, oglReplaceFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::UseTexturesClick(TObject *Sender)
{
  if (!dllOpen) return;

  useTexturesFlag = (UseTextures->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, useTexturesFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::UseWhiteBaseClick(TObject *Sender)
{
  if (!dllOpen) return;

  useWhiteBaseFlag = (UseWhiteBase->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_WHITE_BASE, useWhiteBaseFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::AlwaysShowCameraCursorClick(
      TObject *Sender)
{
  if (!dllOpen) return;

  alwaysShowCameraCursorFlag = (AlwaysShowCameraCursor->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, alwaysShowCameraCursorFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::ReversePanClick(TObject *Sender)
{
  if (!dllOpen) return;

  reversePanFlag = (ReversePan->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_REVERSE_PAN_DIRECTION, reversePanFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LightsForm::ReverseZoomClick(TObject *Sender)
{
  if (!dllOpen) return;

  reverseZoomFlag = (ReverseZoom->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_REVERSE_ZOOM_DIRECTION, reverseZoomFlag);
}
//---------------------------------------------------------------------------
